
function Conn(url) {
    
    if (typeof WebSocket == 'undefined') return false;
    if (this === (function(){return this;})()) return new Conn(url);

    var me = this;
    var ws = null;
    var ws_url = url;
    var reconn_handler = null;
    var reconn_timmer = null;
    var event_handlers = {};

    // 連線
    function connect(_url){

        if (arguments.length) ws_url = _url;
        try {
            ws = new WebSocket(ws_url);
        } catch (e) {
            console.log(e.stack);
        }

        return ws;
    };

    // 關閉連線
    function close() {
        var prev_ws = ws;
        if (ws && (ws.readyState === WebSocket.OPEN || ws.readyState == WebSocket.CONNECTING)) {
            ws.close();
            ws = null;
        }

        return prev_ws;
    };

    // 連線是否存在
    this.isAlive = function(){
        console.log(ws, ws && ws.readyState, (new Error).stack);
        return ws && ws.readyState === WebSocket.OPEN;
    };

    // 事件綁定
    this.on = function(type, handler){

        if ( ! ws) throw new Error('還沒建立連線');
        if ( ! event_handlers[type]) event_handlers[type] = [];
        event_handlers[type].push(handler);
        ws.addEventListener(type, handler);

        return me;
    };

    this.off = function(type, handler){
        
        if (event_handlers[type]) {
            var arr = event_handlers[type];
            for (var i = arr.length; i >= 0; i--) {
                if (arr[i] === handler) arr.splice(i, 1);
            }
        }
        
        if (ws) {
            ws.removeEventListener(type, handler);
        }

        return me;
    };

    this.trigger = function(e){
        if (ws) {
            ws.dispatchEvent(e);
        }
    };

    // 訊息發送
    this.emit = function(msg){
        if ( ! me.isAlive()) throw new Error('還沒建立連線');
        if (typeof msg != 'string') msg = JSON.stringify(msg);
        return ws.send(msg);
    };

    // 綁定斷線重連
    this.reconnect = function(seconds, time){

        var origin_t = time;
        seconds = parseInt(seconds, 10) || 30;

        if (reconn_timmer) reconn_timmer = clearInterval(reconn_timmer);

        reconn_handler = function(){
        
            reconn_timmer = setInterval(function(){
                var new_ws; 
                --time;
                if ( ! me.isAlive()) {
                
                    if (time === 0 && reconn_timmer) reconn_timmer = clearInterval(reconn_timmer);
                    
                    close();
                    rebind(new_ws = connect());
                    bind_reconn(new_ws);
                } else {
                    if (reconn_timmer) reconn_timmer = clearInterval(reconn_timmer);
                }

                console.log('reconn', me.isAlive());

            }, seconds * 1000);
        };
        
        bind_reconn(ws);

        // TODO: 兩個方法往外抽
        function rebind(ws) {
            for (var type in event_handlers) {
                var handlers = event_handlers[type];
                for (var i = 0; i < handlers.length; i++) {
                    ws.addEventListener(type, handlers[i]);
                }
            }
        }

        function bind_reconn(ws) {
            ws.addEventListener('open', function(){
                time = origin_t;
                ws.addEventListener('close', function(){
                    reconn_handler && reconn_handler();
                });
            });
        }

    };

    connect(url);

    this.debug = function(){
        console.log({
            event_handlers: event_handlers,
            reconn_handler: reconn_handler,
            reconn_timmer: reconn_timmer
        });
    };
};

module.exports = Conn; 
